% LICENSE AGREEMENT
% - - - - - - - - -
% Copyright (c) 2018, Stilianos Louca
% All rights reserved.
%
% Redistribution and use in source and binary forms, with or without
% modification, are permitted provided that the following conditions are
% met:
%
%    * Redistributions of source code must retain the above copyright
%      notice, this list of conditions and the following disclaimer.
%    * Redistributions in binary form must reproduce the above copyright
%      notice, this list of conditions and the following disclaimer in
%      the documentation and/or other materials provided with the distribution
%    * Neither the name of the original author (Stilianos Louca), nor the names
%      of any other contributors may be used to endorse or promote products derived
%      from this software without specific prior written permission.
%
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
% AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
% IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
% ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
% LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
% CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
% SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
% INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
% CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
% ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
% POSSIBILITY OF SUCH DAMAGE.
%
% - - - - - - - - -
% Tested on MATLAB R2015b.
% - - - - - - - - -
%
% DESCRIPTION
% - - - - - -
% Split N items into R rows and C columns, i.e. such that R*C ≈ N and C:R ≈ optimalRatio.
% minRatio and maxRatio is minimum and maximum allowed C:R ratio (N is potentially increased to make sure this constraint is satisfied)
function [rows, cols] = num2RowsColumns(N, optimalRatio, minRatio, maxRatio)
	while(true)
		[rows, cols] = num2RowsColumns_aux(N, optimalRatio);
		N = N + 1;
		if(not((cols > rows * maxRatio) | (cols < rows * minRatio))) 
			break; 
		end
	end
end


% Auxiliary function used by num2RowsColumns()
function [rows, cols] = num2RowsColumns_aux(N, ratio)
	if(N == 0)
		rows = 0;
		cols = 0;
	end
	rows0 = sqrt(N/ratio); 
	cols0 = sqrt(N*ratio);
	
	rowst=1; colst=N;
	factor = max(1.0,floor(rows0));
	while(factor>=1)
		if(mod(N,factor) == 0)
			rows = factor; 
			cols = N / factor; 
			break;
		end
		factor = factor - 1;
	end
	dist = abs(rows - rows0) + abs(cols - cols0);
	
	factor = max(1.0,floor(cols));
	while(factor>=1)
		if(mod(N,factor) == 0)
			rowst = factor; colst = N / factor; 
			break;
		end
		factor = factor - 1;
	end
	if((abs(rowst - rows0) + abs(colst - cols0)) < dist)
		rows = rowst; 
		cols = colst;
	end
end
